from WordGameNode import WordGameNode
from WordGameNode import read_dictionary
from collections import deque
import sys


def bfs(start, goal):
    # We will start here, so the list of nodes to do is the start
    todo = deque([start])
    visited = set()
    num_searches = 0

    while todo:
        # Get (and remove) first element in the list (using the list as a queue)
        item = todo.popleft()
        num_searches += 1

        if item.name == goal.name:
            return num_searches, item
        elif goal in todo:
            item = todo.popleft()
            while item != goal:
                item = todo.popleft()
            return num_searches, item
        elif goal not in todo:
            # Keep searching.
            visited.add(item)  # Remember that we've been here
            for child in item.get_children():
                if child not in visited and child not in todo:
                    todo.append(child)

    return num_searches, None  # no route to goal


def main(args):
    if len(args) == 3:
        start_word = args[1]
        goal_word = args[2]
    else:
        start_word = input("Enter the start word: ")
        goal_word = input("Enter the goal word: ")
        # start_word = "moon"
        # goal_word = "star"
    assert len(start_word) == len(goal_word)

    # This reads the dictionary and updates the valid_words global variable for WordGameNode.
    read_dictionary("words.txt", len(start_word))
    start = WordGameNode(start_word)
    goal = WordGameNode(goal_word)

    num_searches, end = bfs(start, goal)  # Do the breadth first search
    print(num_searches)
    if end is None:
        print("There is no path from {0} to {1}".format(start, goal))
    else:
        print(end.get_path())


if __name__ == "__main__":
    main(sys.argv)
